<p class="logo">
  <a href="{{ pathto(master_doc) }}">
    <img class="logo" src="{{ pathto('_static/requests-sidebar.jpg', 1) }}" title="https://kennethreitz.org/tattoos" />
  </a>
</p>

<p>
  <iframe src="https://ghbtns.com/github-btn.html?user=psf&repo=requests&type=watch&count=true&size=large"
    allowtransparency="true" frameborder="0" scrolling="0" width="200px" height="35px"></iframe>
</p>
<link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/docsearch.js@2/dist/cdn/docsearch.min.css" />
<style>
  .algolia-autocomplete {
    width: 100%;
    height: 1.5em
  }

  .algolia-autocomplete a {
    border-bottom: none !important;
  }

  #doc_search {
    width: 100%;
    height: 100%;
  }
</style>
<input id="doc_search" placeholder="Search the doc" autofocus />
<script type="text/javascript" src="https://cdn.jsdelivr.net/npm/docsearch.js@2/dist/cdn/docsearch.min.js" onload="docsearch({
  apiKey: 'f177061e2354c50a97bfc635e827ffab',
  indexName: 'python-requests',
  inputSelector: '#doc_search',
  debug: false // Set debug to true if you want to inspect the dropdown
})" async></script>

<p>
  Requests is an elegant and simple HTTP library for Python, built for
  human beings.
</p>
<p>Sponsored by <strong><a href="https://www.govcert.lu">CERT Gouvernemental - GOVCERT.LU</a></strong>.</p>

<script async type="text/javascript"
  src="//cdn.carbonads.com/carbon.js?zoneid=1673&serve=CKYI5K3E&placement=pythonrequestsorg"
  id="_carbonads_js"></script>

<h3>Stay Informed</h3>
<p>Receive updates on new releases and upcoming projects.</p>

<p><iframe src="https://ghbtns.com/github-btn.html?user=kennethreitz&type=follow&count=false" allowtransparency="true"
    frameborder="0" scrolling="0" width="200" height="20"></iframe></p>

<h3>Useful Links</h3>
<ul>
  <li><a href="https://requests.readthedocs.io/en/latest/community/recommended/">Recommended Packages and Extensions</a>
  </li>

  <p></p>

  <li><a href="https://github.com/psf/requests">Requests @ GitHub</a></li>
  <li><a href="https://pypi.org/project/requests/">Requests @ PyPI</a></li>
  <li><a href="https://github.com/psf/requests/issues">Issue Tracker</a></li>
  <li><a href="https://requests.readthedocs.io/en/latest/community/updates/#software-updates">Release History</a></li>
</ul>


<h3>Translations</h3>

<ul>
  <li><a href="https://requests.readthedocs.io/">English</a></li>
  <li><a href="https://fr.python-requests.org/">French</a></li>
  <li><a href="https://de.python-requests.org/">German</a></li>
  <li><a href="https://jp.python-requests.org/">Japanese</a></li>
  <li><a href="https://cn.python-requests.org/">Chinese</a></li>
  <li><a href="https://pt.python-requests.org/">Portuguese</a></li>
  <li><a href="https://it.python-requests.org/">Italian</a></li>
  <li><a href="https://es.python-requests.org/">Spanish</a></li>
</ul>

<div id="native-ribbon">
</div>
